<?php

/**
 * @file
 * Provides Git Commit administrative interfaces.
 */

/**
 * Form builder for the "clear commit history" page.
 */
function gitcommit_clear_history_form($form, &$form_state) {
  if (isset($form_state['storage']['delete_confirm'])) {
    if (isset($form_state['storage']['delete_time'])) {
      $args = array('@time' => $form_state['storage']['delete_time']);
      $question = t('Are you sure you want to delete all commits performed before @time?', $args);
    }
    else {
      $question = t('Are you sure you want to delete all commits?');
    }
    $form = confirm_form($form, $question, current_path());
  }
  else {
    $form['fieldset_time'] = array(
      '#title' => t('Specify time'),
      '#type' => 'fieldset',
    );
    $form['fieldset_time']['delete_op'] = array(
      '#title' => t('Clear commits of which time period?'),
      '#type' => 'radios',
      '#required' => TRUE,
      '#options' => array(
        'all' => t('All times'),
        'before_time' => t('Before a specific time')
      ),
      '#default_value' => 'all',
    );

    $form['fieldset_time']['delete_time'] = array(
      '#title' => t('Clear commits performed before:'),
      '#type' => 'date_popup',
      '#date_year_range' => '-3:0',
      '#date_format' => 'Y-m-d H:i:s',
      '#date_timezone' => date_default_timezone(),
      '#states' => array(
        'invisible' => array(
          ':input[name="delete_op"]' => array('value' => 'all'),
        ),
        'required' => array(
          ':input[name="delete_op"]' => array('value' => 'before_time'),
        ),
      ),
    );

    $info = entity_get_info('gitcommit');
    $options = array();
    foreach ($info['bundles'] as $bundle => $bundle_info) {
      $options[$bundle] = $bundle_info['label'];
    }
    $form['fieldset_types'] = array(
      '#title' => t('Specify types'),
      '#type' => 'fieldset',
    );
    $form['fieldset_types']['delete_types'] = array(
      '#title' => t('Clear commits of types:'),
      '#type' => 'checkboxes',
      '#required' => TRUE,
      '#options' => $options,
    );

    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Clear history'),
    );
  }

  return $form;
}

/**
 * Submit handler.
 */
function gitcommit_clear_history_form_submit($form, &$form_state) {
  if (!empty($form_state['storage']['delete_confirm'])) {
    $values = $form_state['storage'];

    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'gitcommit')
      ->entityCondition('bundle', $values['delete_types'], 'IN');

    // Delete from a specific time.
    if (empty($values['delete_all'])) {
      $date = new DateTime($values['delete_time']);
      $query->propertyCondition('time', $date->format('U'), '<');
    }

    $result = $query->execute();

    $count = 0;
    if (isset($result['gitcommit'])) {
      // Clear history.
      gitcommit_delete_multiple(array_keys($result['gitcommit']));
      $count = sizeof($result['gitcommit']);
    }
    $msg = format_plural($count, '1 commit has been deleted.', '@count commits have been deleted.');
    drupal_set_message($msg);

    // Back to initial form.
    $values['delete_confirm'] = FALSE;
  }
  else {
    // Set confirmation form.
    $form_state['storage']['delete_confirm'] = TRUE;
    $form_state['rebuild'] = TRUE;
    $values = $form_state['values'];

    // Store values.
    if (isset($values['delete_types'])) {
      $form_state['storage']['delete_types'] = $values['delete_types'];
    }
    if ($values['delete_op'] == 'all') {
      $form_state['storage']['delete_all'] = TRUE;
    }
    else {
      $form_state['storage']['delete_time'] = $values['delete_time'];
    }
  }
}
