<?php

/**
 * Git Commit Reference configuration form.
 */
function gitcommit_reference_config_form($form, &$form_state) {
  $options = array();
  foreach (entity_get_info() as $entity_type => $info) {
    $options[$entity_type] = $info['label'];
  }
  $form['fieldset_entity_types'] = array(
    '#title' => t('Entity types to reference'),
    '#type' => 'fieldset',
  );

  $form['#entity_types_old'] = variable_get('gitcommit_reference_entity_types', array());

  $form['fieldset_entity_types']['gitcommit_reference_entity_types'] = array(
    '#title' => t('Which entity types do you want to reference?'),
    '#type' => 'checkboxes',
    '#options' => $options,
    '#default_value' => $form['#entity_types_old'],
  );

  $form['fieldset_regex'] = array(
    '#title' => t('Regular expression'),
    '#type' => 'fieldset',
  );

  $desc = t('Regular expression used to extract ids of internal entities from the commit message,');
  $desc .= ' ' . t('e.g. #\d+ will catch all integer numbers right after a # character.');
  $options = array('!here' => l('here', 'http://en.wikipedia.org/wiki/Regular_expression'));
  $desc .= ' ' . t('More information about regex !here.', $options);
  $desc .= ' ' . t('Leave blank to do not extract anything.');

  $form['fieldset_regex']['gitcommit_reference_regex'] = array(
    '#title' => t('Regex'),
    '#type' => 'textfield',
    '#description' => $desc,
    '#default_value' => variable_get('gitcommit_reference_regex', '#\d+'),
  );
  $form['#submit'][] = 'gitcommit_reference_config_form_submit';

  return system_settings_form($form);
}

/**
 * Validate handler.
 */
function gitcommit_reference_config_form_validate($form, &$form_state) {
  // User cannot remove bundles that have content in the database.
  $blocked_entities = array(); 
  foreach ($form_state['values']['gitcommit_reference_entity_types'] as $entity_type => $flag_new) {
    $flag_old = !empty($form['#entity_types_old'][$entity_type]);
    if ($flag_old && !$flag_new) {
      $query = new EntityFieldQuery();
      $count = $query->entityCondition('entity_type', 'gitcommit')
        ->entityCondition('bundle', $entity_type)
        ->count()
        ->execute();

      if ($count) {
        $blocked_entities[] = $form['gitcommit_reference_entity_types']['#options'][$entity_type];
      }
    }
  }

  if (!empty($blocked_entities)) {
    $msg = format_plural(
      sizeof($blocked_entities),
      'There are commits of type @type in the database. You need first remove them.',
      'There are commits of types @types in the database. You need first remove them.',
      array('@types' => implode(', ', $blocked_entities))
    );

    form_set_error('gitcommit_reference_entity_types', $msg);
  }
}

/**
 * Submit handler.
 */
function gitcommit_reference_config_form_submit($form, &$form_state) {
  foreach ($form_state['values']['gitcommit_reference_entity_types'] as $entity_type => $flag_new) {
    $flag_old = !empty($form['#entity_types_old'][$entity_type]);

    // Insert a new commit type.
    if ($flag_new && !$flag_old) {
      // Consequently, we need to create an entity reference field.
      $field_info = field_info_instance('gitcommit', 'gitcommit_reference_' . $entity_type, $entity_type);
      if (empty($field_info)) {
        $field_name = 'gitcommit_reference_' . $entity_type;
        $field = array(
          'field_name' => $field_name,
          'type' => 'entityreference',
          'settings' => array(
            'target_type' => $entity_type,
            'handler_settings' => array('target_bundles' => NULL),
          ),
          'cardinality' => FIELD_CARDINALITY_UNLIMITED,
          'translatable' => FALSE,
        );
        field_create_field($field);

        // Attach the company reference field to employee instance
        $instance = array(
          'label' => 'Git Commit Reference',
          'field_name' => $field_name,
          'entity_type' => 'gitcommit',
          'bundle' => $entity_type,
          'settings' => array(
            'target_type' => $entity_type,
            'handler_settings' => array('target_bundles' => NULL),
          ),
        );
        field_create_instance($instance);
      }
    }
    // Remove a commit type.
    elseif ($flag_old && !$flag_new) {
      // Consequently, we need to remove the entity reference field.
      $field_info = field_info_instance('gitcommit', 'gitcommit_reference_' . $entity_type, $entity_type);
      if (!empty($field_info)) {
        field_delete_field($field_info['field_name']);
      }
    }
  }

  // Refreshing commit entity information.
  $query = db_select('cache', 'c')
    ->fields('c', array('cid'))
    ->condition('cid', db_like('entity_info:') . '%', 'LIKE')
    ->execute();

  while ($cid = $query->fetchField()) {
    cache_clear_all($cid, 'cache');
  }
}
